function [x,map]=bmp8rd(filename);
%BMP8RD Citeste un fisier BMP (Microsoft Windows Bitmap) de 8 bpp.
%	[x,map]=BMP8RD('filename') citeste fisierul 'filename' si intoarce
%	planul indexat de culoare X. Daca nu se precizeaza extensia,
%	se ia implicit '.bmp'.
%
%       Creat de C. Vertan la 20 iul. 1995
%       Ultima modificare C.Vertan 20 iul. 1995


if (nargin~=1)
	help bmp8rd
	error('Necesita un nume de fisier ca argument !');
end;

if (isstr(filename)~=1)
	help bmp8rd	
	error('Argumentul trebuie sa fie un sir !');
end;

if (isempty(findstr(filename,'.'))==1)
	filename=[filename,'.bmp'];
end;

fid=fopen(filename,'rb','l');
if (fid==-1)
	error(['Error opening ',filename,' for input.']);
end;

bfType=fread(fid,2,'char');
setstr(bfType);
if (bfType~=['B';'M'])
	fclose(fid);
	error('Not a BMP file.');
end;

bfSize=fread(fid,1,'long');
bfReserved1=fread(fid,1,'short');
bfReserved2=fread(fid,1,'short');
bfOffBits=fread(fid,1,'long');

biSize=fread(fid,1,'long');
if (biSize~=40)
	fclose(fid);
	error('Not a MS Windows Device Independent Bitmap BMP file.');
end;

biWidth=fread(fid,1,'long');
biHeight=fread(fid,1,'long');
biPlanes=fread(fid,1,'short');
biBitCount=fread(fid,1,'short');
biCompression=fread(fid,1,'long');
biSizeImage=fread(fid,1,'long');
biXPels=fread(fid,1,'long');
biYPels=fread(fid,1,'long');
biClrUsed=fread(fid,1,'long');
biClrImportant=fread(fid,1,'long');

if (biClrUsed==0)
	nColors=2.^biBitCount;
else
	nColors=biClrUsed;
end;

if (biClrUsed>256)
	map=[];	% 24-bit images have no colormap
else
	map=fread(fid,4*nColors,'uchar');
	map=reshape(map,4,nColors);
	map=map';
	map=map(:,1:3);
	map=fliplr(map);
end;

map=map./255;

rest=rem(biWidth,4);
if (rest~=0)
	rest=4-rest;
end

if (biBitCount==8)
  temp=fread(fid,[biWidth+rest,biHeight],'uchar');
  x=rot90(temp(1:biWidth,:))+1;
  else
	help bmp8rd
	error('Acesta nu e un BMP cu 8 bpp !');
end;


fclose(fid);


