%ENHANCE     Realizeaza imbunatatirea imaginii prin transformari punctuale
%
% [OUT,TRANSFORM,MAP]=ENHANCE(IN,TIP,PARAM,L);
% Semnificatia parametrilor de intrare este:
% IN este imaginea de intrare, avand L nivele de gri (implicit 256)
%
% TIP este tipul de transformare, care poate fi:
%'liniar' : modificarea liniara a contrastului, cu segmente de dreapta
%'exponential' : modificare exponentiala a contrastului (expandare)
%'logaritmic' : modificare logaritmica a contrastului (compandare)
%'neliniar' : modificare a contrastului prin functie putere
%'punct fix' : modificare neliniara a contrastului pe 2 intervale
%'slicing' : transformare de decupare a unui interval de nivele de gri
%'clipping' : transformare de lipire a unui interval de nivele de gri
%'histeq' : egalizare de histograma
%
% PARAM este matricea parametrilor ce sunt necesari anumitor transformari:
%PARAM = [a b Fa Fb] daca TIP este 'liniar'
%PARAM = [r] daca TIP este 'neliniar'
%PARAM = [r s] daca TIP este 'punct fix'
%PARAM = [a b] daca TIP este 'slicing' sau 'clipping'
%PARAM nu trebuie specificat pentru celelalte transformari ('histeq',
%'exponential' si 'logaritmic').
%
%Semnificatia parametrilor de iesire este:
% OUT este imaginea de iesire
% TRANSFORM este vectorul transformarii nivelelor de gri
% MAP este noua tabela de culoare
%
% Creata de M. Ciuc		la 01.10.2000
% Ultima modificare: C. Vertan	la 03.05.2001

function [img_out,val_noi,map_nou]=enhance(img_in,tip,alti_par,L);

if nargin < 2,
   help enhance
   error('Nu ati furnizat suficienti parametri!');
end

if (exist('L','var')~=1)
   L=256;
end

val_noi=ones(1,L);
[nr_lin nr_col]=size(img_in);
img_out=zeros(nr_lin*nr_col,1);


if (nargin < 3)&(strcmp(tip,'liniar')|strcmp(tip,'neliniar')|strcmp(tip,'punct fix') ...
      |strcmp(tip,'slicing')|strcmp(tip,'clipping'))
   help enhance
   error('Nu ati furnizat parametrii necesari transformarii!');
end

if strcmp(tip,'liniar'),
   a=alti_par(1);
   b=alti_par(2);
   Fa=alti_par(3);
   Fb=alti_par(4);
   
   val_noi(1:a)=round(((Fa-1)*(1:a)+(a-Fa))/(a-1));
   val_noi(a+1:b)=round(((Fb-Fa)*(a+1:b)+b*Fa-a*Fb)/(b-a));
   val_noi(b+1:L)=round(((L-Fb)*(b+1:L)+256*(Fb-b))/(L-b));
   
elseif strcmp(tip,'logaritmic'),
   val_noi=1+round((L-1)*log(1:L)/log(L));


elseif strcmp(tip,'exponential'),
   val_noi=1+round((exp(0:log(L)/(L-1):log(L))-1));


elseif strcmp(tip,'neliniar'),
   r=alti_par(1);
   val_noi=1+round((L-1)*(0:1/(L-1):1).^r);
   
   
elseif strcmp(tip,'punct fix'),
   r=alti_par(1);
   s=alti_par(2);
   val_noi(1:s+1)=1+round(s*((0:1/s:1).^r));
   val_noi(s+2:L)=s+1+round((L-1-s)*(1-(1:-1/(L-2-s):0).^r));


elseif strcmp(tip,'clipping'),
   a=alti_par(1);
   b=alti_par(2);
   val_noi([1:a-1,b+1:L])=ones(1,L-1+a-b);
   val_noi(a:b)=a:b;


elseif strcmp(tip,'slicing'),
   a=alti_par(1);
   b=alti_par(2);
   val_noi(a:b)=L*ones(1,b-a+1);

elseif strcmp(tip,'histeq'),
   for i=1:L
      hist_in(i)=sum(img_in(:)==i);
   end
   H=cumsum(hist_in/(nr_lin*nr_col));
   val_noi=round(H*(L-1)+1);

else
   help enhance
   error('Parametrul TIP nu corespune');
end

img_out=reshape(val_noi(img_in),nr_lin,nr_col);
map_nou=(val_noi-1)'*ones(1,3)/(L-1);
