%LFILT L-filtru (combinatie liniara a statisticilor de ordine)
%
% OUT=LFILT(IN,WW)
% OUT este imaginea filtrata (indexata, cu nivele de gri)
% IN este imaginea de prelucrat (indexata, cu nivele de gri)
% WW este vectorul coeficientilor filtrului; trebuie sa aiba
% 1 linie si 9 coloane (corespunzand unei filtrari intr-o fereastra
% centrata de 3x3 pixeli). In mod implicit, coeficientii sunt alesi
% astfel ca filtrarea sa fie de tip median. Suma coeficientilor trebuie
% sa fie 1 pentru o filtrare de netezire sau 0 pentru o filtrarea de
% tip derivativ. Implementarea este realizata in domeniul spatial prin
% algoritmul ferestrei glisante.
%
% Scrisa de C. Vertan la 22.03.1997
% Ultima modificare C. Vertan la 13.04.2001

function [out]=lfilt(in,ww)

if (exist('ww','var')==0)
   ww=zeros(1,9);
   ww(5)=1;
end

[hww,www]=size(ww);
if ((hww~=1)&(www~=9))
   error('Vectorul de ponderi nu este linie cu 9 coloane !');
end

tol=10^(-5);
sum_coef=sum(ww);
if round(sum_coef-1)>tol & round(sum_coef)>tol,
   error('Nucleul de filtrare nu corespunde unei filtrari corecte !');
end

[h,w]=size(in);
out=in;

for i=2:h-1
 for j=2:w-1
  temp=in(i-1:i+1,j-1:j+1);
  temp=sort(temp(:));
  out(i,j)=fix(ww*temp);
 end
end