%LINFILT Filtru liniar (convolutie cu nuclee mici)
%
% OUT=LINFILT(IN,NUCLEU,SW)
% OUT este imaginea filtrata (indexata, cu nivele de gri)
% IN este imaginea de prelucrat (indexata, cu nivele de gri)
% NUCLEU este masca de filtrare, matrice cu dimenisuni impare.
% Suma valorilor din matricea NUCLEU trebuie sa fie 1 pentru o
% filtrare de netezire sau 0 pentru o filtrare derivativa.
% SW permite selectia unei filtrari in domeniul spatial (SW=0)
% prin algoritmul ferestrei glisante, sau in frecventa (SW=1)
% prin transformate Fourier rapide. In mod implicit se alege SW=1.
%
% Scrisa de M. Ciuc la 17.10.1998
% Ultima modificare C. Vertan la 31.01.2001

function [out]=linfilt(in,mask,sw)

if (exist('sw','var')==0)
   sw=1;
end

if nargin < 2,
   help linfilt 
   error('Nu ati furnizat numarul minim necesar de parametri!');
end

[li ci]=size(in);
[ln cn]=size(mask);

if ~(ln-2*floor(ln/2) | cn-2*floor(cn/2)),
   help linfilt
   error('Dimensiunile nucleului de filtrare trebuie sa fie numere impare!');
end

tol=10^(-5);
sum_coef=sum(mask(:));
%if round(sum_coef-1)>tol & round(sum_coef)>tol,
 %  error('Nucleul de filtrare nu corespunde unei filtrari corecte !');
%end

ln=floor(ln/2);
cn=floor(cn/2);

if (sw==1)
  out=(round(real(ifft2(fft2(in).*fft2(mask,li,ci)))));
  out=[out out;out out];
  out=out(1+ln:li+ln,1+cn:ci+cn);
  %out=max(1,min(256,out));
else
  in=[zeros(ln,ci+2*cn);zeros(li,cn) in zeros(li,cn);zeros(ln,ci+2*cn)];
  for i=1+ln:li+ln,
    for j=1+cn:ci+cn,
      a=in(i-ln:i+ln,j-cn:j+cn);
      out(i-ln,j-cn)=(round(a(:)'*mask(:)));
    end
  end
  %out=max(1,min(256,out));   
end

if (sum_coef)
   out=max(1,min(256,out));
end
   
   