function out = nagao(in,nriter)

%
%      Performs Nagao filtering on grayscale images
%         ImOut = nagao(ImIn,NbIter)
%               ImIn is the input image
%               ImOut is the output image
%               NbIter is the number of times the filter is applied (default = 1)
%

if ~exist('in','var')
    error('You must specify an input image!');
end

if ~exist('nriter','var')
    nriter = 1;
end

[lin col]=size(in);

out = in;

mask = [1,2,6,7,8,12,13;...
        6,7,11,12,13,16,17;...
        12,13,16,17,18,21,22;...
        13,17,18,19,22,23,24;...
        13,14,18,19,20,24,25;...
        9,10,13,14,15,19,20;...
        3,4,8,9,10,13,14;...
        2,3,4,7,8,9,13;...
    ]';


for k = 1:nriter
    for i = 3:lin-2,
        for j = 3:col-2,
            temp = in(i-2:i+2,j-2:j+2);
            [dummy,poz]=min(var(temp(mask)));
            out(i,j)=mean(temp(mask(:,poz)));
        end
    end
end