function [rest,Q] = rest_fic(fs,h,c,sigma)

%    REST_FIC		realizeaza restaurarea unei imagini cu filtrul invers 
%    cu constrangeri
%        [IMG_REST,Q] = REST_FIC(IMG_DEGR,NUCLEU_DEGR,NUCLEU_TS,LAMBDA)
%           - IMG_REST este imaginea restaurata
%           - Q este functia de transfer a filtrului de restaurare
%           - IMG_DEGR este imaginea degradata
%           - NUCLEU_DEGR este nucleul de degradare
%           - NUCLEU_TS este nucleul trece-sus
%           - SIGMA este dispersia zgomotului 
%

if nargin ~= 4,
   help rest_fic
   error('Ati furnizat un numar gresit de parametri de intrare');
end

[lin col]=size(fs);

Er = sigma^2*lin*col;
lambda = 1e-10;
pas = lambda;
H1 = fft2(h,lin,col);
H = abs(H1).^2;
C = abs(fft2(c,lin,col)).^2;
Fs = abs(fft2(fs)).^2.*(C.^2);
dim = lin*col;
if sigma~=0,
   while 1,
      E = sum(Fs(:)./(C(:)+lambda*H(:)).^2)/dim-Er;
      if abs(E) < 1e-2,
         break;
      end
      while 1,
         Epas = sum(Fs(:)./(C(:)+(lambda+pas)*H(:)).^2)/dim-Er;
         if Epas ~= E,
            break;
         end
         pas = pas*10;
      end 
      m = (Epas - E)/pas;
      lambda = lambda - E/m;
   end
   Q = lambda*conj(H1)./(C+lambda*H);
else
   Q = 1./H1;
end
rest = real(ifft2(fft2(fs,lin,col).*Q));
Q = fftshift(Q);