function [rest,Q] = rest_wiener(fs,h,sigma)

%     REST_WIENER        realizeaza restaurarea unei imagini cu filtrul
%                        de restaurare Wiener
%
%        [IMG_REST,Q] = REST_FIC(IMG_DEGR,NUCLEU_DEGR,DISPERSIE)
%           - IMG_REST este imaginea restaurata
%           - Q este functia de transfer a filtrului de restaurare Wiener
%           - IMG_DEGR este imaginea degradata
%           - NUCLEU_DEGR este nucleul de degradare
%           - DISPERSIE este dispersia zgomotului
%

if nargin ~= 3,
   help rest_wiener,
   error('Ati furnizat un numar gresit de parametri la intrare');
end


[lin col]=size(fs);


fs = fft2(fs);
H = fft2(h,lin,col);

if sigma~=0,
   Qr = sigma^2*lin*col*ones(lin,col);
   Qf = max(abs(fs).^2-Qr,0);
   Q = conj(H).*Qf./(abs(H).^2.*Qf+Qr);
else,
   Q = 1./H;
end

rest = real(ifft2(fs.*Q));
Q = fftshift(Q);
