% WHISTO Calculeaza histograma ponderata a unei imagini
%
% H=HISTO(IN,P,MASK)
% H este vectorul histogramei imaginii
% HCUM este vectorul histogramei cumulative
% IN este imaginea cu nivele de gri (1 - 256) carei i se calculeaza
% histograma ponderata. Ponderarea se face dupa factorul
% (1+|x|)^P. x este determinat de o masca (MASK) de filtrare liniara
% de dimensiune 3x3; daca nu se da, se ia implicit un laplacian.
% Daca puterea P nu se da, se ia IMPLICIT valoarea -1.
%
% Creata de C. Vertan la 20.02.97
% Ultima modificare la   22.02.98
function [h,hcum]=histo(in,p,mask)

if (exist('p')==0)
 p=-1;
end
if (exist('mask')==0)
 mask=[0 -1 0;-1 4 -1;0 -1 0];
end


[hh,w]=size(in);
h=zeros(1,256);

in=[zeros(1,w+2);zeros(hh,1) in zeros(hh,1);zeros(1,w+2)];

for i=2:hh+1
 for j=2:w+1
  temp=in(i-1:i+1,j-1:j+1);
  la=(temp(:))'*mask(:);
  h(in(i,j))=h(in(i,j))+(1+abs(la))^p;
 end
end

h=h/(w*hh);
hcum=cumsum(h);